>From d4a06a167ba7b9848fac358462cc28e3fea43242 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Jan=20Urba=C5=84ski?= <wulczer@wulczer.org>
Date: Sun, 2 Jan 2011 11:14:43 +0100
Subject: [PATCH 03/16] Fix an error when a set-returning function fails halfway through the execution.

If the function using yield to return rows fails halfway, the iterator
stays open and subsequent calls to the function will resume reading
from it. The fix is to unref the iterator and set it to NULL if there
has been an error.
---
 src/pl/plpython/plpython.c |    8 ++++++++
 1 files changed, 8 insertions(+), 0 deletions(-)

diff --git a/src/pl/plpython/plpython.c b/src/pl/plpython/plpython.c
index 64a8a4a..f0932bf 100644
--- a/src/pl/plpython/plpython.c
+++ b/src/pl/plpython/plpython.c
@@ -1145,6 +1145,14 @@ PLy_function_handler(FunctionCallInfo fcinfo, PLyProcedure *proc)
 		Py_XDECREF(plargs);
 		Py_XDECREF(plrv);
 
+		/*
+		 * If there was an error the iterator might have not been exhausted
+		 * yet. Set it to NULL so the next invocation of the function will
+		 * start the iteration again.
+		 */
+		Py_XDECREF(proc->setof);
+		proc->setof = NULL;
+
 		PG_RE_THROW();
 	}
 	PG_END_TRY();
-- 
1.7.2.3

